#  ----------------------------------------------------------------------------
#            Functions for computing band-structured matrices
#  ----------------------------------------------------------------------------
Kfunband.1eSC <- function(x, gamval, omeval)
{
   #  This evaluates the reproducing kernel associated with the operator
   #  L = (gamval D + D^2) * (omeval^2 I + D^2)
   #  at the array X of argument values
   
   #  Arguments:
   #  X      ... array of length N of argument values
   #  GAMVAL ... value of parameter GAMVAL
   #  OMEVAL ... value of parameter OMEVAL
   
   #  Return:
   #  Band-structured matrix of size N by 9
   
   #  There are four cases, associated with a set of spanning functions:
   #  gamval   and omeval nonzero:  1, exp(gamval*x), sin(omval*x), cos(omval*x) 
   #  gamval 0 and omeval nonzero:  1, x            , sin(omval*x), cos(omval*x) 
   #  gamval nonzero and omeval 0:  1, x, x^2, exp(gamval*x), 
   #  gamval and omeval 0:          1, x, x^2, x^3
   
   #  The last case is not handled; instead a message indicates that one 
   #  should use the module Pspline
   
   #  Because values are needed in only for the diagonal and 
   #  first 8 off-diagonal bands, the function returns a 
   #  length(x) by 9 band-structured array 
  
   if (gamval == 0 & omeval == 0) 
     stop ("Both GAMVAL and OMEVAL 0, use Pspline module instead.")
     
   if (abs(gamval) > 20) 
     stop ("Value of GAMVAL exceeds 20")
     
   n     <- length(x)
   RKmat <- matrix(0,n,9)
   ier   <- 0

   if (gamval == 0) {
      result <- .Fortran("kfunband_1tsc", 
                  as.integer(n), as.double(x), as.double(omeval),
                  as.double(RKmat), as.integer(ier))
      ier <- result[[5]]
      if (ier != 0) stop("IER nonzero.")
      RKmat <- matrix(result[[4]],n,9)
   }

   if (omeval == 0) {
      result <- .Fortran("kfunband_1tt2e", 
                  as.integer(n), as.double(x), as.double(gamval),
                  as.double(RKmat), as.integer(ier))
      ier <- result[[5]]
      if (ier != 0) stop("IER nonzero.")
      RKmat <- matrix(result[[4]],n,9)
   }

   if (omeval != 0 & gamval != 0) {
     result <- .Fortran("kfunband_1esc", 
                  as.integer(n),     as.double(x), 
                  as.double(gamval), as.double(omeval),
                  as.double(RKmat),  as.integer(ier))

     ier <- result[[6]]
     if (ier != 0) stop("IER nonzero.")
     RKmat <- matrix(result[[5]],n,9)
   }
   
   return(RKmat)
}

#  ----------------------------------------------------------------------------
 
Kfunband.1SC <- function(x, omeval)
{
   #  This evaluates the reproducing kernel associated with the operator
   #  L = omeval^2 D + D^3
   #  at the array X of argument values
   
   #  Because Kfunband.1ST must be called from within the Lspline module with
   #  only the argument X, omeval must be given a value
   #  prior to the call to Lspline
   
   #  Because values are needed in only for the diagonal and 
   #  first 6 off-diagonal bands, the function returns a 
   #  length(x) by 7 band-structured array 
  
   if (omeval == 0) 
     stop ("OMEVAL 0, use Pspline module instead.")
     
   n     <- length(x)
   RKmat <- matrix(0,n,7)
   ier   <- 0

   result <- .Fortran("kfunband_1sc", 
                  as.integer(n), as.double(x), as.double(omeval),
                  as.double(RKmat), as.integer(ier))
   ier <- result[[5]]
   if (ier != 0) stop("IER nonzero.")
   RKmat <- matrix(result[[4]],n,7)
   return(RKmat)
}

# ----------------------------------------------------------------------------

Kfunband.1E <-  function(x, gamval)
{
  #  computes reproducing kernel for operator L = -gamma*D + D^2
  #  Arguments:
  #  X      ... array of length N of argument values
  #  GAMVAL ... value of parameter GAMVAL
   
  #  Return:
  #  Band-structured matrix of size N by 5
   
  if (gamval == 0) 
     stop ("GAMVAL 0, use Pspline module instead.")
  if (abs(gamval) > 20) 
     stop ("Value of GAMVAL exceeds 20")
  n <- length(x)
  RKmat  <- matrix(0,n,5)
  ier    <- 0
  result <- .Fortran("kfunband_1e", as.integer(n), as.double(x), 
                  as.double(gamval),
                  as.double(RKmat), as.integer(ier))
  ier <- result[[5]]
  if (ier != 0) stop("IER nonzero.")
  RKmat <- matrix(result[[4]],n,5)
  return(RKmat)
}
 
# ----------------------------------------------------------------------------

Kfunband.1tE <-  function(x, gamval)
{
  #  computes reproducing kernel for operator L = -gamma*D^2 + D^3
  #  Arguments:
  #  X      ... array of length N of argument values
  #  GAMVAL ... value of parameter GAMVAL
   
  #  Return:
  #  Band-structured matrix of size N by 7
   
  if (gamval == 0) 
     stop ("GAMVAL 0, use Pspline module instead.")
  if (abs(gamval) > 20) 
     stop ("Value of GAMVAL exceeds 20")
  n <- length(x)
  RKmat  <- matrix(0,n,7)
  ier    <- 0
  result <- .Fortran("kfunband_1te", as.integer(n), as.double(x), 
                  as.double(gamval),
                  as.double(RKmat), as.integer(ier))
  ier <- result[[5]]
  if (ier != 0) stop("IER nonzero.")
  RKmat <- matrix(result[[4]],n,7)
  return(RKmat)
}

# ----------------------------------------------------------------------------

Kfunband.1tt2E <-  function(x, gamval)
{
  #  computes reproducing kernel for operator L = -gamma*D^3 + D^4
  #  Arguments:
  #  X      ... array of length N of argument values
  #  GAMVAL ... value of parameter GAMVAL
   
  #  Return:
  #  Band-structured matrix of size N by 9
   
  if (gamval == 0) 
     stop ("GAMVAL 0, use Pspline module instead.")
  if (abs(gamval) > 20) 
     stop ("Value of GAMVAL exceeds 20")
  n <- length(x)
  RKmat  <- matrix(0,n,9)
  ier    <- 0
  result <- .Fortran("kfunband_1tt2e", as.integer(n), as.double(x), 
                  as.double(gamval),
                  as.double(RKmat), as.integer(ier))
  ier <- result[[5]]
  if (ier != 0) stop("IER nonzero.")
  RKmat <- matrix(result[[4]],n,9)
  return(RKmat)
}

#  ----------------------------------------------------------------------------
#            Functions for computing complete matrices
#  ----------------------------------------------------------------------------

Kfunfull.1eSC <- function(x1, x2, gamval, omeval)
{
   #  This evaluates the reproducing kernel associated with the operator
   #  L = (gamval D + D^2) * (omval^2 I + D^2)
   #  at the arrays X1 and X2 of argument values
   
   #  Arguments:
   #  X1     ... array of length N1 of argument values
   #  X2     ... array of length N2 of argument values
   #  GAMVAL ... value of parameter GAMVAL
   #  OMEVAL ... value of parameter OMEVAL
   
   #  Return:
   #  Matrix of size N1 by N2
      
   #  There are four cases, associated with a set of spanning functions:
   #  gamval   and omeval nonzero:  1, exp(gamval*x), sin(omval*x), cos(omval*x) 
   #  gamval 0 and omeval nonzero:  1, x            , sin(omval*x), cos(omval*x) 
   #  gamval nonzero and omeval 0:  1, x, x^2, exp(gamval*x), 
   #  gamval and omeval 0:          1, x, x^2, x^3
   
   #  The last case is not handled; instead a message indicates that one 
   #  should use the module Pspline
   
   if (gamval == 0 & omeval == 0) 
     stop ("Both GAMVAL and OMEVAL 0, use Pspline module instead.")
     
   if (abs(gamval) > 20) 
     stop ("Value of GAMVAL exceeds 20")
     
   n1 <- length(x1)
   n2 <- length(x2)
   RKmat <- matrix(0,n1,n2)
   ier   <- 0

   if (gamval == 0) {
      result <- .Fortran("kfunfull_1tsc", 
                  as.integer(n1),   as.integer(n2), 
                  as.double(x1),    as.double(x2), as.double(omeval),
                  as.double(RKmat), as.integer(ier))
      ier <- result[[7]]
      if (ier != 0) stop("IER nonzero.")
      RKmat <- matrix(result[[6]],n1,n2)
    }

   if (omeval == 0) {
      result <- .Fortran("kfunfull_1tt2e", 
                  as.integer(n1),   as.integer(n2), 
                  as.double(x1),    as.double(x2), as.double(gamval),
                  as.double(RKmat), as.integer(ier))
      ier <- result[[7]]
      if (ier != 0) stop("IER nonzero.")
      RKmat <- matrix(result[[6]],n1,n2)
    }

    if (omeval != 0 & gamval != 0) {
      result <- .Fortran("kfunfull_1esc", 
                  as.integer(n1),    as.integer(n2),
                  as.double(x1),     as.double(x2),
                  as.double(gamval), as.double(omeval),
                  as.double(RKmat),  as.integer(ier))

      ier <- result[[8]]
      if (ier != 0) stop("IER nonzero.")
      RKmat <- matrix(result[[7]],n1,n2)
   }
   
   return(RKmat)
}

#  ----------------------------------------------------------------------------

Kfunfull.1SC <- function(x1, x2, omeval)
{
   #  This evaluates the reproducing kernel associated with the operator
   #  L = omeval^2 D + D^3
   #  at the arrays X1 and X2 of argument values
   
   #  Because Kfunfull.1ST must be called from within the Lspline module with
   #  only the argument X, OMEVAL must be given values
   #  prior to the call to Lspline
   
   #  The function returns a length(X1) by length(X2) array 
  
   if (omeval == 0) 
     stop ("OMEVAL 0, use Pspline module instead.")
     
   n1 <- length(x1)
   n2 <- length(x2)
   RKmat <- matrix(0,n1,n2)
   ier   <- 0

   result <- .Fortran("kfunfull_1tsc", 
                  as.integer(n1),   as.integer(n2), 
                  as.double(x1),    as.double(x2), as.double(omeval),
                  as.double(RKmat), as.integer(ier))
   ier <- result[[7]]
   if (ier != 0) stop("IER nonzero.")
   RKmat <- matrix(result[[6]],n1,n2)   
   return(RKmat)
}

#  ----------------------------------------------------------------------------

Kfunfull.1E <-  function(x1, x2, gamval)
{
   #  computes reproducing kernel K2 for operator L = -gamval*D + D^2
   
   #  Arguments:
   #  X1     ... array of length N1 of argument values
   #  X2     ... array of length N2 of argument values
   #  GAMVAL ... value of parameter GAMVAL
   
   #  Return:
   #  Matrix of size N1 by N2
  
   if (gamval == 0) 
     stop ("Both GAMVAL and OMEVAL 0, use Pspline module instead.")
     
   if (abs(gamval) > 20) 
     stop ("Value of GAMVAL exceeds 20")
     
   n1 <- length(x1)
   n2 <- length(x2)
   RKmat  <- matrix(0,n1,n2)
   ier    <- 0
   result <- .Fortran("kfunfull_1e", 
                  as.integer(n1), as.integer(n2),
                  as.double(x1),  as.double(x2), as.double(gamval),
                  as.double(RKmat), as.integer(ier))

   ier <- result[[7]]
   if (ier != 0) stop("IER nonzero.")
   RKmat <- matrix(result[[6]],n1,n2)
   return(RKmat)
}
 
#  ----------------------------------------------------------------------------

Kfunfull.1tE <-  function(x1, x2, gamval)
{
   #  computes reproducing kernel K2 for operator L = -gamval*D^2 + D^3
   
   #  Arguments:
   #  X1     ... array of length N1 of argument values
   #  X2     ... array of length N2 of argument values
   #  GAMVAL ... value of parameter GAMVAL
   
   #  Return:
   #  Matrix of size N1 by N2
  
   if (gamval == 0) 
     stop ("Both GAMVAL and OMEVAL 0, use Pspline module instead.")
     
   if (abs(gamval) > 20) 
     stop ("Value of GAMVAL exceeds 20")
     
   n1 <- length(x1)
   n2 <- length(x2)
   RKmat  <- matrix(0,n1,n2)
   ier    <- 0
   result <- .Fortran("kfunfull_1te", 
                  as.integer(n1), as.integer(n2),
                  as.double(x1),  as.double(x2), as.double(gamval),
                  as.double(RKmat), as.integer(ier))

   ier <- result[[7]]
   if (ier != 0) stop("IER nonzero.")
   RKmat <- matrix(result[[6]],n1,n2)
   return(RKmat)
}

#  ----------------------------------------------------------------------------

Kfunfull.1tt2E <-  function(x1, x2, gamval)
{
   #  computes reproducing kernel K2 for operator L = -gamval*D^3 + D^4
   
   #  Arguments:
   #  X1     ... array of length N1 of argument values
   #  X2     ... array of length N2 of argument values
   #  GAMVAL ... value of parameter GAMVAL
   
   #  Return:
   #  Matrix of size N1 by N2
  
   if (gamval == 0) 
     stop ("Both GAMVAL and OMEVAL 0, use Pspline module instead.")
     
   if (abs(gamval) > 20) 
     stop ("Value of GAMVAL exceeds 20")
     
   n1 <- length(x1)
   n2 <- length(x2)
   RKmat  <- matrix(0,n1,n2)
   ier    <- 0
   result <- .Fortran("kfunfull_1tt2e", 
                  as.integer(n1), as.integer(n2),
                  as.double(x1),  as.double(x2), as.double(gamval),
                  as.double(RKmat), as.integer(ier))

   ier <- result[[7]]
   if (ier != 0) stop("IER nonzero.")
   RKmat <- matrix(result[[6]],n1,n2)
   return(RKmat)
}
 
 

